import events
from pygame.sprite import Sprite
from pygame import Rect
import pygame
from utils import *
import math

DRAGTWIST_RATIO = 80
DRAGSTRETCH_RATIO = 20

def ChangeAndDirtyAttr( attrName ):
	def Setter(self, val):
		print 'setting ', attrName, val
		if val == self.__dict__[attrName]:
			return
		self.__dict__[attrName] = val
		self._imageDirty = True
	def Getter(self):
		return self.__dict__[attrName]

	return property(Getter, Setter)


class Dot(Sprite):
	red = 0
	def __init__(self, radius):
		global red
		self.radius = radius
		self.image = TransSurface( self.radius*2, self.radius*2 )
		if DEBUG:
			color = (Dot.red,Dot.red,255,255)
			Dot.red += 22
		else:
			color = (255,255,255,255)
		pygame.draw.circle( self.image, color, (10,10), self.radius )
		self.rect = self.image.get_rect()
		#print 'self.rect', self.rect

class Marble(Sprite):
	def __init__(self, radius=10, width=40):
		events.AddListener(self)

		self.radius = radius
		self.width = width

		self.stretch = 40
		self.stretchFloor = 40
		self.stretchCeiling = 200

		self.twist = 0
		self.twistFloor = 0
		self.twistCeiling = 3

		self.dots = [
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		             Dot(self.radius),
		            ]
		self.rect = self.MakeRect()
		self.CalcOriginPoint()

		self.active = True
		self._focused = False
		self._dragging = False
		self.dragStartPoint = None

		self._imageDirty = True
		self._cachedImage = None
	
	def CalcOriginPoint(self):
		self.originPoint = HalfwayBetween( self.dots[0].rect.center,
		                                   self.dots[1].rect.center )

	def GetImage(self):
		if not self._imageDirty:
			return self._cachedImage
		#print 'get image called.'
		ds = self.dots
		pairs = zip( ds[::2], ds[1::2] )
		#pairs = ds[0:2], ds[2:4], ds[4:6], ds[6:8]
		stretchStep = self.stretch/len(pairs)
		twistStep = self.twist/len(pairs)

		for i, pair in enumerate(pairs):
			pair[0].rect.center = (self.radius,self.radius)
			pair[1].rect.center = (self.radius,self.radius)

			pair[1].rect.centery = pair[0].rect.centery + self.width

			pair[0].rect.centerx += i*stretchStep
			pair[1].rect.centerx += i*stretchStep

			thisTwist = math.sin(math.pi*twistStep*i)*self.width
			thisTwist = abs(thisTwist)
			#print 'percentage pi *', twistStep, i
			#print 'percentage', math.sin(math.pi*twistStep*i)
			#print 'thisTwist', thisTwist
			pair[0].rect.centery += thisTwist
			pair[1].rect.centery -= thisTwist
		self.CalcOriginPoint()

		newImg = TransSurface( *self.rect.size )

		for dot in ds:
			#print 'blitting dot at', dot.rect, 'on', self.rect
			newImg.blit( dot.image, dot.rect )

		if DEBUG:
			if self.focused:
				color = (255,100,100)
			else:
				color = (200,0,0)
			pygame.draw.rect( newImg, color, self.rect, 2 )
			pygame.draw.circle( newImg, color, self.originPoint, 3 )

		self._cachedImage = newImg
		#self._imageDirty = False
		return newImg

	def SetImage(self, value):
		print "oh really fool?"

	image = property(GetImage, SetImage)
	focused = ChangeAndDirtyAttr('_focused')
	dragging = ChangeAndDirtyAttr('_dragging')

	def MakeRect(self):
		'''Returns a rect with dimensions sufficient to contain all
		of the dots.
		'''
		ds = self.dots
		newRect = Rect( ds[0] )
		newRect.height = ds[1].rect.bottom - ds[0].rect.top
		newRect.width = ds[-1].rect.right - ds[0].rect.left
		return newRect
		
	def update(self, timeChange=1):
		newRect = self.MakeRect()
		newRect.topleft =  self.rect.topleft
		self.rect = newRect
		#print 'updated rect to', self.rect
		if self.dragging:
			pass
		else:
			self.update_UnwindStep(timeChange)
	
	def update_UnwindStep(self, timeChange):
		self.StretchMore( -2.8 )

	def On_MouseDown(self, event):
		if not self.rect.collidepoint( event.pos ):
			self.focused = False
			return

		self.focused = True
		self.dragging = True
		self.dragStartPoint = event.pos

	def On_MouseUp(self, event):
		if not self.focused:
			return

		self.dragging = False

	def StretchMore( self, furtherStretch ):
		self.stretch += furtherStretch/DRAGSTRETCH_RATIO
		self.stretch = Clamp( self.stretch, 
		                      self.stretchFloor, self.stretchCeiling )
		twistAmt = (self.stretch - self.stretchFloor) /\
		           (self.stretchCeiling - self.stretchFloor)
		self.twist = (self.twistCeiling - self.twistFloor) * twistAmt
		

	def On_MouseMotion(self, event):
		#print 'handling mousemouion'
		if not self.focused and self.rect.collidepoint( event.pos ):
			self.focused = True
			self._imageDirty = True
		if not self.dragging:
			return

		currentDragDistance = Distance( event.pos, self.dragStartPoint )
		originToDragStart = Distance( self.originPoint, self.dragStartPoint )
		originToDragPos = Distance( self.originPoint, event.pos )
		if originToDragPos < originToDragStart:
			#reverse it if dragging towards the origin
			currentDragDistance *= -1
		self.StretchMore( currentDragDistance )

			
		

class Maoble:
	def __init__(self):
		self.elasticBand = ElasticBand()
		self.velocity = [0,0]

	def update( self, timeChange ):
		self.elasticBand.update( timeChange )
		angle = GetAngle( VectorMultiply( self.velocity,(-1,-1)) )

	def Start(self):
		events.AddListener(self)

	def On_LevelStartedEvent(self):
		#TODO: do something here.
		self.avatar = Marble()
		events.Fire("AvatarBirthEvent", self.avatar)
