import events
import pygame
from pygame.locals import *
import data
from logging import info as log_info, debug as log_debug
from effects import eManager

from pygame.sprite import RenderUpdates


class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, name, pos):
		pygame.sprite.Sprite.__init__(self)

		self.image = data.pngs[name]

		self.rect = self.image.get_rect()
		self.rect.topleft = pos


class GameScreen:
	def __init__(self, screen, display, level=1):
		self.screen = screen
		self.display = display

		self.effectsGroup = RenderUpdates()
		self.avatarGroup = RenderUpdates()
		self.bg = None

		self.isDone = False
		self.isStarted = False
		self.nextDisplayer = None
		self.keydown = False

		self.level = level

	def Start( self ):
		self.bg = pygame.Surface( (800,600) )
		off_black = (40,110,0)
		self.bg.fill( off_black )

		events.AddListener( self )

		self.isDone = False
		self.isStarted = True

		self.screen.blit( self.bg, (0,0) )

		self.display.flip()

		eManager.Start( self.effectsGroup )
		#data.oggs['game_song'].play( -1 )
		events.Fire( "LevelStartedEvent" )

	def Finish( self ):
		self.effectsGroup.empty()
		self.avatarGroup.empty()
		events.RemoveListener( self )
		self.isDone = True
		self.isStarted = False
		data.oggs['game_song'].fadeout( 1200 )

	def On_AvatarBirth(self, avatar):
		log_debug( 'avatar being born' )
		self.avatarGroup.add( avatar )
		self.avatar.rect.center = (300,300)
		


	def On_AvatarDeath(self, event):
		log_debug( 'setting death cutscene' )
		self.nextDisplayer = cutscenes.Death(self)

	def On_LevelCompletedEvent(self, event):
		nextLevel = self.level + 1
		log_debug( 'setting next cutscene %d'%nextLevel )
		self.nextDisplayer = cutscenes.GetNext(self, nextLevel)
		#print self, 'just set nextD to ', self.nextDisplayer
		self.Finish()

	def On_KeyDown(self, event):
		if event.key == K_q:
			self.keydown = True

	def On_KeyUp(self, event):
		if event.key == K_q:
			self.keydown = False
			self.Finish()

	def Tick(self, timeChange):
		if not self.isStarted:
			log_info( 'starting' )
			self.Start()

		#clear
		self.effectsGroup.clear( self.screen, self.bg )
		self.avatarGroup.clear( self.screen, self.bg )


		#update
		events.ConsumeEventQueue()
		self.avatarGroup.update( timeChange ) #must go first!
		self.effectsGroup.update( timeChange )
		events.ConsumeEventQueue()

		#draw
		changedRects = self.avatarGroup.draw(self.screen)
		changedRects += self.effectsGroup.draw(self.screen)
		self.display.update( changedRects )


